#
# $Source: /home/nlfm/Working/Zircon/Released/lang/RCS/suomi.tcl,v $
# $Date: 1999/11/26 09:36:42 $
# $Revision: 1.18.1.1 $
#
# ----------------------------------------------------------------------
#   AUTHOR:  Lindsay Marshall <lindsay.marshall@newcastle.ac.uk>
# ----------------------------------------------------------------------
# Copyright 1996 The University of Newcastle upon Tyne (see COPYRIGHT)
# ======================================================================
#
array set ztrans {
    abort	Keskeytä
    accept	Hyväksy
    action	Toiminta
    actions	Toiminnat
    active      Aktiivinen
    admin       Ylläpito
    all         Kaikki
    anonymous	Nimetön
    append      Lisää
    apply       Käytä
    away	Poissaolo
    back	Takaisin
    background  Tausta
    ban		Kielto
    brb		BRB
    busy	Kiireinen
    buttons	Painikkeet
    call	Kutsu
    cancel	Peruuta
    channel	Kanava
    channels	Kanavat
    chanop	KanavaOp
    chat	Keskustelu
    clear	Tyhjennä
    clientinfo  Klienttitiedot
    clone	Klooni
    close	Sulje
    configure	Määritä
    connect	Kytkeydy
    conversation Keskustelu
    crypt	Salaus
    ctcp	CTCP
    current     Viimeisin
    dcc		DCC
    debug	Korjaa
    default	Oletus
    delete	Poista
    dismiss	Poistu
    draw	Piirrä
    dump	Hävitä
    edit	Muokkaa
    empty	Tyhjennä
    error	Virhe
    exec	{Suorita komento}
    favourites	Suosikit
    file        Tiedosto
    filename    {Tiedoston nimi}
    filter	Suodatin
    finger	Finger
    flush	Huuhtele
    font        Fontti
    foreground  Edusta
    friends     Ystävät
    get		Saada
    height      Korkeus
    help	Apu
    history	Historia
    hostname	{Koneen nimi}
    icon        Ikoni
    ignore      {Älä huomioi}
    ignores     {Älä huomioi}
    info	Tiedot
    information Informaatio
    invisible	Näkymättömyys
    invite	Kutsu
    irc		IRC
    ircname	{Irc nimi}
    ircnames    {Irc nimet}
    join	Liity
    jump	Hyppää
    keep	Pidä
    key		Avain
    kick	Potki
    kill	Tapa
    leave	Poistu
    load	Kuormitus
    log		Loki
    lost        Kadonnut
    limit	Rajoitus
    link        Linkki
    links       Linkit
    list	Lista
    lusers      Käyttäjät
    map		Kartta
    message	Viesti
    messages	Viestit
    mode	Tila
    moderated	Hillitty
    monitor	Tarkkaile
    motd        {Päivän sanoma}
    name	Nimi
    names	Nimet
    new		Uusi
    nickname	Lisänimi
    nicknames   Lisänimet
    no          Ei
    nocase	{Pienet kirjaimet}
    noshare     {Älä jaa}
    notice	Ilmoitus
    notify	Huomioi
    offer	Tarjoa
    ok		Hyväksy
    open	Avaa
    oper        Operoi
    operator	Operaattori
    other	Muu
    parameter	Parametri
    parameters	Parametrit
    password	Salasana
    pattern	Kuvio
    people	Ihmiset
    plugin	{Plug In}
    port	Portti
    print       Tulosta
    private	Yksityinen
    query	Kysely
    quiet	Vaiti
    quit	Poistu
    reconnecting	{Kytkeytyy uudelleen}
    refresh	Virkistä
    register	Kirjaudu
    rehash      Virkistä
    reject	Hylkää
    rejoin	{Liity uudelleen}
    restart     {Käynnistä uudelleen}
    revert      Palaa
    save	Tallenna
    script	Skripti
    scroll	Vieritä
    secret	Salainen
    send	Lähetys
    server	Serveri
    servers	Serverit
    service	Palvelu
    services	Palvelut
    set	        Aseta
    share       Jaa
    shutdown    Sammuta
    signoff     {Lähde pois}
    silence     Hiljaisuus
    sound       Ääni
    speak       Puhu
    stats       Statistiikka
    tell        Kerro
    text        Teksti
    time        Aika
    trace       Jäljitä
    type        Tyyppi
    topic       Teema
    unban       {Perua kielto}
    unload	Pura
    user	Käyttäjä
    userinfo    Käyttäjätiedot
    users	Käyttäjät
    version     Versio
    view	Katso
    who		Kuka
    whois	{Kuka on}
    whowas	{Kuka oli}
    width       Leveys
    windows	Ikkunat
    yes         Kyllä
}
#
# Compounds of the above that are no language dependent
#
array set ztrans "
    bankick	{$ztrans(ban)+$ztrans(kick)}
"

