# Tcl package index file, version 1.1
# This file is generated by the "pkg_mkIndex" command
# and sourced either when an application starts up or
# by a "package unknown" script.  It invokes the
# "package ifneeded" command to set up package-related
# information so that packages will be loaded automatically
# in response to "package require" commands.  When this
# script is sourced, the variable $dir must contain the
# full path name of this file's directory.

package ifneeded macfile 1.18.8.0 [list source [file join $dir macfile.tcl]]
package ifneeded macintosh 1.18 [list source [file join $dir mac.tcl]]
package ifneeded support 1.18 [list source [file join $dir support.tcl]]
package ifneeded unix 1.18 [list source [file join $dir unix.tcl]]
package ifneeded unixfile 1.18.7.5 [list source [file join $dir file75.tcl]]
package ifneeded unixfile 1.18.7.6 [list source [file join $dir file76.tcl]]
package ifneeded unixfile 1.18.8.0 [list source [file join $dir file80.tcl]]
package ifneeded unixfile 1.18.8.1 [list source [file join $dir file81.tcl]]
package ifneeded unixfile 1.18.8.2 [list source [file join $dir file82.tcl]]
package ifneeded unixfile 1.18.8.3 [list source [file join $dir file83.tcl]]
package ifneeded unixfile 1.18.8.4 [list source [file join $dir file84.tcl]]
package ifneeded windows 1.18 [list source [file join $dir windows.tcl]]
package ifneeded winfile 1.18.8.0 [list source [file join $dir winfile.tcl]]
